/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "H323-MESSAGES.h"
#include "rtxsrc/rtxDiag.h"

#define MAXMSGLEN 1024

static ASN1OBJID gProtocolID = {
   6, { 0, 0, 8, 2250, 0, 2 }
} ;

int main (int argc, char** argv)
{
   H225AliasAddress aliasAddress;
   H225TransportAddress_ipAddress ipAddress;
   H225TransportAddress transportAddress;
   H225RegistrationConfirm registrationConfirm;
   H225RasMessage rasMessage;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN], *msgptr;
   OSBOOL       aligned = TRUE, trace = TRUE, verbose = FALSE;
   FILE*        fp;
   const char*  filename = "message.dat";
   int		i, len, stat, tvalue = 2;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-t")) tvalue = atoi(argv[++i]);
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -t  choice tag value to encode\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */

   if (rtInitContext (&ctxt) != 0) {
      rtxErrPrint (&ctxt);
      return -1;
   }
   rtxSetDiag (&ctxt, verbose);
   pu_setBuffer (&ctxt, msgbuf, sizeof(msgbuf), aligned);
   pu_setTrace (&ctxt, trace);

   /* Populate structure */

   memset (&registrationConfirm, 0, sizeof(registrationConfirm));
   registrationConfirm.m.terminalAliasPresent = 1;
   registrationConfirm.m.gatekeeperIdentifierPresent = 1;

   registrationConfirm.requestSeqNum = 1;
   rtSetOID (&registrationConfirm.protocolIdentifier, &gProtocolID);

   /* Call signal address */

   ipAddress.ip.numocts = 4;
   ipAddress.ip.data[0] = 0xc0;
   ipAddress.ip.data[1] = 0xa8;
   ipAddress.ip.data[2] = 0x00;
   ipAddress.ip.data[3] = 0x01;
   ipAddress.port = 1720;

   memset (&transportAddress, 0, sizeof(transportAddress));
   transportAddress.t = T_H225TransportAddress_ipAddress;
   transportAddress.u.ipAddress = &ipAddress;
   rtxDListInit (&registrationConfirm.callSignalAddress);
   rtxDListAppend 
      (&ctxt, &registrationConfirm.callSignalAddress, &transportAddress);

   /* Terminal alias */

   memset (&aliasAddress, 0, sizeof(aliasAddress));
   aliasAddress.t = T_H225AliasAddress_h323_ID;
   rtCToBMPString (&ctxt, "anH323ID", &aliasAddress.u.h323_ID, 0);
   rtxDListInit (&registrationConfirm.terminalAlias);
   rtxDListAppend (&ctxt, &registrationConfirm.terminalAlias, &aliasAddress);

   /* Gatekeeper identifier */

   rtCToBMPString (&ctxt, "aGatekeeperID", 
                   &registrationConfirm.gatekeeperIdentifier, 0);

   /* Endpoint identifier */

   rtCToBMPString (&ctxt, "anEndpointID", 
                   &registrationConfirm.endpointIdentifier, 0);

   memset (&rasMessage, 0, sizeof(rasMessage));
   rasMessage.t = T_H225RasMessage_registrationConfirm;
   rasMessage.u.registrationConfirm = &registrationConfirm;

   /* Encode */

   if ((stat = asn1PE_H225RasMessage (&ctxt, &rasMessage)) == 0) {
      if (trace) {
         printf ("Encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         pu_hexdump (&ctxt);
         printf ("Binary dump:\n");
         pu_bindump (&ctxt, "rasMessage");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return -1;
   }
  
   msgptr = pe_GetMsgPtr (&ctxt, &len);

   /* Write the encoded message out to the output file */

   if (fp = fopen (filename, "wb")) {
      fwrite (msgptr, 1, len, fp);
      fclose (fp);
   }
   else {
      printf ("Error opening %s for write access\n", filename);
      return -1;
   }

   rtFreeContext (&ctxt);

   return 0;

}
